(ns com.aurellem.gb.stats
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def pokemon-1-stats-start-address 0xD18B)

(defn pokemon-stats-address [poke-num]
  (+ pokemon-1-stats-start-address
     (* poke-num pokemon-record-width)))

(def stats-record-size 11)

(defn read-stats
  ([^SaveState state poke-num]
     (let [start (pokemon-stats-address poke-num)
           [level
            hp-h
            hp-l
            attack-h
            attack-l
            defense-h
            defense-l
            speed-h
            speed-l
            special-h
            special-l]
           (subvec (vec (memory state))
                   start (+ start stats-record-size ))]
       {:level level
        :hp (glue-bytes hp-h hp-l)
        :attack (glue-bytes attack-h attack-l)
        :defense (glue-bytes defense-h defense-l)
        :speed (glue-bytes speed-h speed-l)
        :special (glue-bytes special-h special-l)}))
  ([poke-num]
     (read-stats @current-state poke-num)))
          
(defn give-stats
  ([^SaveState state poke-num new-stats]
     (let [new-stats* (merge (read-stats state poke-num)
                             new-stats)
           raw-stats
           (flatten
            [(:level new-stats*)
             (disect-bytes-2 (:hp new-stats*))
             (disect-bytes-2 (:attack new-stats*))
             (disect-bytes-2 (:defense new-stats*))
             (disect-bytes-2 (:speed new-stats*))
             (disect-bytes-2 (:special new-stats*))])]
       (set-memory-range state (pokemon-stats-address poke-num)
                         raw-stats)))
  ([poke-num new-stats]
     (give-stats @current-state poke-num new-stats)))
